package dev.fyloz.colorrecipesexplorer.service

import com.nhaarman.mockitokotlin2.mock
import com.nhaarman.mockitokotlin2.spy
import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.repository.RecipeStepRepository
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import kotlin.test.assertTrue

class RecipeStepServiceTest :
  AbstractModelServiceTest<RecipeStep, RecipeStepService, RecipeStepRepository>() {
    override val repository: RecipeStepRepository = mock()
    override val service: RecipeStepService = spy(RecipeStepServiceImpl(repository))

    override val entity: RecipeStep = recipeStep(id = 0L, message = "message")
    override val anotherEntity: RecipeStep = recipeStep(id = 1L, message = "another message")

    // validateStepsCollection()

    @Test
    fun `validateStepsCollection() throws an InvalidStepsPositionsException when the position of the first step of the given groupInformation is not 1`() {
        withSteps(
          mutableSetOf(
            recipeStep(id = 0L, position = 0),
            recipeStep(id = 1L, position = 1),
            recipeStep(id = 2L, position = 2),
            recipeStep(id = 3L, position = 3)
          )
        ) {
            val exception = assertThrows<InvalidStepsPositionsException> {
                service.validateGroupInformationSteps(this)
            }

            assertTrue { exception.errors.count() == 1 }
            assertTrue { exception.errors.first().type == INVALID_FIRST_STEP_POSITION_ERROR_CODE }
        }
    }

    @Test
    fun `validateStepsCollection() throws an InvalidStepsPositionsException when steps positions are duplicated in the given groupInformation`() {
        withSteps(
          mutableSetOf(
            recipeStep(id = 0L, position = 1),
            recipeStep(id = 1L, position = 2),
            recipeStep(id = 2L, position = 2),
            recipeStep(id = 3L, position = 3)
          )
        ) {
            val exception = assertThrows<InvalidStepsPositionsException> {
                service.validateGroupInformationSteps(this)
            }

            assertTrue { exception.errors.count() == 1 }
            assertTrue { exception.errors.first().type == DUPLICATED_STEPS_POSITIONS_ERROR_CODE }
        }
    }

    @Test
    fun `validateStepsCollection() throws an InvalidStepsPositionsException when there is a gap between steps positions in the given groupInformation`() {
        withSteps(
          mutableSetOf(
            recipeStep(id = 0L, position = 1),
            recipeStep(id = 1L, position = 2),
            recipeStep(id = 2L, position = 4),
            recipeStep(id = 3L, position = 5)
          )
        ) {
            val exception = assertThrows<InvalidStepsPositionsException> {
                service.validateGroupInformationSteps(this)
            }

            assertTrue { exception.errors.count() == 1 }
            assertTrue { exception.errors.first().type == GAP_BETWEEN_STEPS_POSITIONS_ERROR_CODE }
        }
    }

    private fun withSteps(steps: MutableSet<RecipeStep>, test: RecipeGroupInformation.() -> Unit) {
        recipeGroupInformation(
          group = employeeGroup(id = 0L),
          steps = steps
        ) {
            test()
        }
    }
}
